@@ -12,8 +12,6 @@
 
 """ ISO metadata parser """
 
-import warnings
-
 from owslib.etree import etree
 from owslib import util
 from owslib.namespaces import Namespaces
@@ -29,12 +27,48 @@ def get_namespaces():
 
 namespaces = get_namespaces()
 
+def testFirstCharOrAnchor(md,xpath):
+    """ function which checks if the first matching element is either charstring or anchor, if anchor, returns {name, url}, else {name} """
+    r = {'name': '', 'url': ''}
+    val = md.find(util.nspath_eval(xpath+'/gco:CharacterString', namespaces))
+    r['name'] = util.testXMLValue(val)
+    if r['name'] in [None,'']:    
+        val = md.find(util.nspath_eval(xpath+'/gmx:Anchor', namespaces))
+        if val is not None:
+            r['name'] = util.testXMLValue(val)
+            r['url'] = val.attrib.get(util.nspath_eval('xlink:href', namespaces))
+    return r
+
+def testAllCharOrAnchor(md,xpath,aslist=True):
+    """ 
+    function which checks for each matching element if it is charstring or anchor, returns {name, uri}
+    the aslist parameter indicates to return 2 lists or single list of objects
+    """
+    ro = []
+    for i in md.findall(util.nspath_eval(xpath+'/gco:CharacterString', namespaces)):
+        r = {'name': '', 'url': ''}
+        r['name'] = util.testXMLValue(i)
+        ro.append(r)
+    for i in md.findall(util.nspath_eval(xpath+'/gmx:Anchor', namespaces)):
+        r = {'name': '', 'url': ''}
+        if i is not None:
+            r['name'] = util.testXMLValue(i)
+            r['url'] = i.attrib.get(util.nspath_eval('xlink:href', namespaces)) 
+            ro.append(r)
+    if aslist:
+        return ro
+    else:
+        return {'name': [i.get('name', '') for i in ro],
+                'url': [i.get('url', '') for i in ro]}
+
+
 
 class MD_Metadata(object):
     """ Process gmd:MD_Metadata """
     def __init__(self, md=None):
 
         if md is None:
+            self.md = None
             self.xml = None
             self.identifier = None
             self.parentidentifier = None
@@ -50,15 +84,14 @@ class MD_Metadata(object):
             self.stdver = None
             self.locales = []
             self.referencesystem = None
-            self.identification = None
             self.contentinfo = None
-            self.serviceidentification = None
-            self.identificationinfo = []
+            self.identification = []
             self.contentinfo = []
             self.distribution = None
             self.dataquality = None
             self.acquisition = None
         else:
+            self.md = md
             if hasattr(md, 'getroot'):  # standalone document
                 self.xml = etree.tostring(md.getroot())
             else:  # part of a larger document
@@ -116,38 +149,18 @@ class MD_Metadata(object):
             else:
                 self.referencesystem = None
 
-            # TODO: merge .identificationinfo into .identification
-            warnings.warn(
-                'the .identification and .serviceidentification properties will merge into '
-                '.identification being a list of properties.  This is currently implemented '
-                'in .identificationinfo.  '
-                'Please see https://github.com/geopython/OWSLib/issues/38 for more information',
-                FutureWarning)
+            self.identification = []
 
-            val = md.find(util.nspath_eval('gmd:identificationInfo/gmd:MD_DataIdentification', namespaces))
-            val2 = md.find(util.nspath_eval('gmd:identificationInfo/srv:SV_ServiceIdentification', namespaces))
-
-            if val is not None:
-                self.identification = MD_DataIdentification(val, 'dataset')
-                self.serviceidentification = None
-            elif val2 is not None:
-                self.identification = MD_DataIdentification(val2, 'service')
-                self.serviceidentification = SV_ServiceIdentification(val2)
-            else:
-                self.identification = None
-                self.serviceidentification = None
-
-            self.identificationinfo = []
             for idinfo in md.findall(util.nspath_eval('gmd:identificationInfo', namespaces)):
                 if len(idinfo) > 0:
                     val = list(idinfo)[0]
                     tagval = util.xmltag_split(val.tag)
                     if tagval == 'MD_DataIdentification':
-                        self.identificationinfo.append(MD_DataIdentification(val, 'dataset'))
+                        self.identification.append(MD_DataIdentification(val, 'dataset'))
                     elif tagval == 'MD_ServiceIdentification':
-                        self.identificationinfo.append(MD_DataIdentification(val, 'service'))
+                        self.identification.append(MD_DataIdentification(val, 'service'))
                     elif tagval == 'SV_ServiceIdentification':
-                        self.identificationinfo.append(SV_ServiceIdentification(val))
+                        self.identification.append(SV_ServiceIdentification(val))
 
             self.contentinfo = []
             for contentinfo in md.findall(
@@ -176,6 +189,22 @@ class MD_Metadata(object):
             else:
                 self.acquisition = None
 
+    def get_all_contacts(self):
+        """get all contacts in document"""
+
+        contacts = []
+
+        for ii in self.identification:
+            for iic in ii.contact:
+                contacts.append(iic)
+
+            for ct in ['creator', 'publisher', 'contributor']:
+                iict = getattr(ii, ct)
+                if iict:
+                    contacts.append(iict)
+
+        return list(filter(None, contacts))
+
     def get_default_locale(self):
         """ get default gmd:PT_Locale based on gmd:language """
 
@@ -195,11 +224,14 @@ class PT_Locale(object):
             self.charset = None
         else:
             self.id = md.attrib.get('id')
-            self.languagecode = md.find(
-                util.nspath_eval('gmd:languageCode/gmd:LanguageCode', namespaces)).attrib.get('codeListValue')
-            self.charset = md.find(
-                util.nspath_eval('gmd:characterEncoding/gmd:MD_CharacterSetCode', namespaces)).attrib.get(
-                'codeListValue')
+            self.languagecode = None
+            languagecode = md.find(util.nspath_eval('gmd:languageCode/gmd:LanguageCode', namespaces))
+            if languagecode is not None:
+                self.languagecode = languagecode.attrib.get('codeListValue')
+            self.charset = None
+            charset = md.find(util.nspath_eval('gmd:characterEncoding/gmd:MD_CharacterSetCode', namespaces))
+            if charset is not None:
+                self.charset = charset.attrib.get('codeListValue')
 
 
 class CI_Date(object):
@@ -229,7 +261,9 @@ class CI_ResponsibleParty(object):
 
         if md is None:
             self.name = None
+            self.name_url = None
             self.organization = None
+            self.organization_url = None
             self.position = None
             self.phone = None
             self.fax = None
@@ -242,11 +276,14 @@ class CI_ResponsibleParty(object):
             self.onlineresource = None
             self.role = None
         else:
-            val = md.find(util.nspath_eval('gmd:individualName/gco:CharacterString', namespaces))
-            self.name = util.testXMLValue(val)
 
-            val = md.find(util.nspath_eval('gmd:organisationName/gco:CharacterString', namespaces))
-            self.organization = util.testXMLValue(val)
+            frm = testFirstCharOrAnchor(md,'gmd:individualName')
+            self.name = frm['name']
+            self.name_url = frm['url']
+
+            frm = testFirstCharOrAnchor(md,'gmd:organisationName')
+            self.organization = frm['name']
+            self.organization_url = frm['url']
 
             val = md.find(util.nspath_eval('gmd:positionName/gco:CharacterString', namespaces))
             self.position = util.testXMLValue(val)
@@ -318,27 +355,18 @@ class MD_Keywords(object):
     Class for the metadata MD_Keywords element
     """
     def __init__(self, md=None):
-        warnings.warn(
-            'The .keywords_object attribute will become .keywords proper in the next release. '
-            '.keywords_object is a list of ibstances of the Keyword class. '
-            'See for https://github.com/geopython/OWSLib/pull/765 more details.',
-            FutureWarning)
-
         if md is None:
             self.keywords = []
-            self.keywords_object = []
             self.type = None
             self.thesaurus = None
             self.kwdtype_codeList = 'http://standards.iso.org/ittf/PubliclyAvailableStandards/ISO_19139_Schemas/resources/codelist/gmxCodelists.xml#MD_KeywordTypeCode'
         else:
             self.keywords = []
-            self.keywords_object = []
             val = md.findall(util.nspath_eval('gmd:keyword/gco:CharacterString', namespaces))
             if len(val) == 0:
                 val = md.findall(util.nspath_eval('gmd:keyword/gmx:Anchor', namespaces))
             for word in val:
-                self.keywords.append(util.testXMLValue(word))
-                self.keywords_object.append(Keyword(word))
+                self.keywords.append(Keyword(word))
             self.type = None
             val = md.find(util.nspath_eval('gmd:type/gmd:MD_KeywordTypeCode', namespaces))
             self.type = util.testXMLAttribute(val, 'codeListValue')
@@ -348,8 +376,8 @@ class MD_Keywords(object):
             if val is not None:
                 self.thesaurus = {}
 
-                thesaurus = val.find(util.nspath_eval('gmd:title/gco:CharacterString', namespaces))
-                self.thesaurus['title'] = util.testXMLValue(thesaurus)
+                title = val.find(util.nspath_eval('gmd:title/gco:CharacterString', namespaces))
+                self.thesaurus['title'] = util.testXMLValue(title)
                 self.thesaurus['url'] = None
 
                 if self.thesaurus['title'] is None:  # try gmx:Anchor
@@ -358,12 +386,12 @@ class MD_Keywords(object):
                         self.thesaurus['title'] = util.testXMLValue(t)
                         self.thesaurus['url'] = t.attrib.get(util.nspath_eval('xlink:href', namespaces))
 
-                thesaurus = val.find(util.nspath_eval('gmd:date/gmd:CI_Date/gmd:date/gco:Date', namespaces))
-                self.thesaurus['date'] = util.testXMLValue(thesaurus)
+                date_ = val.find(util.nspath_eval('gmd:date/gmd:CI_Date/gmd:date/gco:Date', namespaces))
+                self.thesaurus['date'] = util.testXMLValue(date_)
 
-                thesaurus = val.find(
+                datetype = val.find(
                     util.nspath_eval('gmd:date/gmd:CI_Date/gmd:dateType/gmd:CI_DateTypeCode', namespaces))
-                self.thesaurus['datetype'] = util.testXMLAttribute(thesaurus, 'codeListValue')
+                self.thesaurus['datetype'] = util.testXMLAttribute(datetype, 'codeListValue')
 
 
 class MD_DataIdentification(object):
@@ -401,8 +429,6 @@ class MD_DataIdentification(object):
             self.graphicoverview = []
             self.contact = []
             self.keywords = []
-            self.keywords_object = []
-            self.keywords2 = []
             self.topiccategory = []
             self.supplementalinformation = None
             self.extent = None
@@ -424,23 +450,48 @@ class MD_DataIdentification(object):
 
             self.uricode = []
             _values = md.findall(util.nspath_eval(
-                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:RS_Identifier/gmd:code/gco:CharacterString',
+                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:MD_Identifier/gmd:code/gco:CharacterString',
                 namespaces))
             _values += md.findall(util.nspath_eval(
-                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:MD_Identifier/gmd:code/gco:CharacterString',
+                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:RS_Identifier/gmd:code/gco:CharacterString',
                 namespaces))
             for i in _values:
                 val = util.testXMLValue(i)
-                if val is not None:
+                if val not in [None,'']:
                     self.uricode.append(val)
+            
+            _values = md.findall(util.nspath_eval(
+                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:MD_Identifier/gmd:code/gmx:Anchor',
+                namespaces))
+            _values += md.findall(util.nspath_eval(
+                'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:RS_Identifier/gmd:code/gmx:Anchor',
+                namespaces))
+            for i in _values:
+                val = util.testXMLValue(i)
+                val1 = i.attrib.get(util.nspath_eval('xlink:href', namespaces))
+                if val1 not in [None,'']:
+                    self.uricode.append(val1)
+                elif val not in [None,'']:
+                    self.uricode.append(val)
+                    
 
             self.uricodespace = []
             for i in md.findall(util.nspath_eval(
                     'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:RS_Identifier/gmd:codeSpace/gco:CharacterString',
                     namespaces)):
                 val = util.testXMLValue(i)
-                if val is not None:
+                if val not in [None,'']:
                     self.uricodespace.append(val)
+            for i in md.findall(util.nspath_eval(
+                    'gmd:citation/gmd:CI_Citation/gmd:identifier/gmd:RS_Identifier/gmd:codeSpace/gmx:Anchor',
+                    namespaces)):
+                val = util.testXMLValue(i)
+                val1 = i.attrib.get(util.nspath_eval('xlink:href', namespaces))
+                if val1 not in [None,'']:
+                    self.uricode.append(val1)
+                elif val not in [None,'']:
+                    self.uricode.append(val)
+
 
             self.date = []
             self.datetype = []
@@ -487,13 +538,9 @@ class MD_DataIdentification(object):
                 if val is not None:
                     self.classification.append(val)
 
-            self.otherconstraints = []
-            for i in md.findall(util.nspath_eval(
-                    'gmd:resourceConstraints/gmd:MD_LegalConstraints/gmd:otherConstraints/gco:CharacterString',
-                    namespaces)):
-                val = util.testXMLValue(i)
-                if val is not None:
-                    self.otherconstraints.append(val)
+            ocs = testAllCharOrAnchor(md,'gmd:resourceConstraints/gmd:MD_LegalConstraints/gmd:otherConstraints', False)
+            self.otherconstraints = ocs['name']
+            self.otherconstraints_url = ocs['url']
 
             self.securityconstraints = []
             for i in md.findall(util.nspath_eval(
@@ -593,50 +640,9 @@ class MD_DataIdentification(object):
                 if val:
                     self.spatialrepresentationtype.append(val)
 
-            warnings.warn(
-                'The .keywords and .keywords2 properties will merge into the '
-                '.keywords property in the future, with .keywords becoming a list '
-                'of MD_Keywords instances. This is currently implemented in .keywords2. '
-                'Please see https://github.com/geopython/OWSLib/issues/301 for more information',
-                FutureWarning)
-
             self.keywords = []
-
-            for i in md.findall(util.nspath_eval('gmd:descriptiveKeywords', namespaces)):
-                mdkw = {}
-                mdkw['type'] = _testCodeListValue(i.find(util.nspath_eval(
-                    'gmd:MD_Keywords/gmd:type/gmd:MD_KeywordTypeCode', namespaces)))
-
-                mdkw['thesaurus'] = {}
-
-                val = i.find(util.nspath_eval(
-                    'gmd:MD_Keywords/gmd:thesaurusName/gmd:CI_Citation/gmd:title/gco:CharacterString', namespaces))
-                mdkw['thesaurus']['title'] = util.testXMLValue(val)
-
-                val = i.find(util.nspath_eval(
-                    'gmd:MD_Keywords/gmd:thesaurusName/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:date/gco:Date',
-                    namespaces))
-                mdkw['thesaurus']['date'] = util.testXMLValue(val)
-
-                val = i.find(util.nspath_eval(
-                    'gmd:MD_Keywords/gmd:thesaurusName/gmd:CI_Citation/gmd:date/gmd:CI_Date/gmd:dateType/gmd:CI_DateTypeCode',
-                    namespaces))
-                mdkw['thesaurus']['datetype'] = util.testXMLAttribute(val, 'codeListValue')
-
-                mdkw['keywords'] = []
-
-                for k in i.findall(util.nspath_eval('gmd:MD_Keywords/gmd:keyword', namespaces)):
-                    val = k.find(util.nspath_eval('gco:CharacterString', namespaces))
-                    if val is not None:
-                        val2 = util.testXMLValue(val)
-                        if val2 is not None:
-                            mdkw['keywords'].append(val2)
-
-                self.keywords.append(mdkw)
-
-            self.keywords2 = []
             for mdkw in md.findall(util.nspath_eval('gmd:descriptiveKeywords/gmd:MD_Keywords', namespaces)):
-                self.keywords2.append(MD_Keywords(mdkw))
+                self.keywords.append(MD_Keywords(mdkw))
 
             self.topiccategory = []
             for i in md.findall(util.nspath_eval('gmd:topicCategory/gmd:MD_TopicCategoryCode', namespaces)):
@@ -712,25 +718,32 @@ class MD_Distribution(object):
     def __init__(self, md=None):
         if md is None:
             self.format = None
+            self.format_url = None
             self.version = None
+            self.version_url = None
+            self.specification = None
+            self.specification_url = None
             self.distributor = []
             self.online = []
             pass
         else:
-            val = md.find(util.nspath_eval(
-                'gmd:distributionFormat/gmd:MD_Format/gmd:name/gco:CharacterString', namespaces))
-            self.format = util.testXMLValue(val)
+            frm = testFirstCharOrAnchor(md,'gmd:distributionFormat/gmd:MD_Format/gmd:name')
+            self.format = frm['name']
+            self.format_url = frm['url']
+            
+            vrs = testFirstCharOrAnchor(md,'gmd:distributionFormat/gmd:MD_Format/gmd:version')
+            self.version = vrs['name']
+            self.version_url = vrs['url']
 
-            val = md.find(util.nspath_eval(
-                'gmd:distributionFormat/gmd:MD_Format/gmd:version/gco:CharacterString', namespaces))
-            self.version = util.testXMLValue(val)
+            spc = testFirstCharOrAnchor(md,'gmd:distributionFormat/gmd:MD_Format/gmd:specification')
+            self.specification = spc['name']
+            self.specification_url = spc['url']
 
             self.distributor = []
             for dist in md.findall(util.nspath_eval('gmd:distributor', namespaces)):
                 self.distributor.append(MD_Distributor(dist))
 
             self.online = []
-
             for ol in md.findall(util.nspath_eval(
                     'gmd:transferOptions/gmd:MD_DigitalTransferOptions/gmd:onLine/gmd:CI_OnlineResource',
                     namespaces)):
@@ -742,6 +755,7 @@ class DQ_DataQuality(object):
     def __init__(self, md=None):
         if md is None:
             self.conformancetitle = []
+            self.conformancetitle_url = []
             self.conformancedate = []
             self.conformancedatetype = []
             self.conformancedegree = []
@@ -750,13 +764,9 @@ class DQ_DataQuality(object):
             self.specificationtitle = None
             self.specificationdate = []
         else:
-            self.conformancetitle = []
-            for i in md.findall(util.nspath_eval(
-                    'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:title/gco:CharacterString',
-                    namespaces)):
-                val = util.testXMLValue(i)
-                if val is not None:
-                    self.conformancetitle.append(val)
+            cts = testAllCharOrAnchor(md,'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:title', False)
+            self.conformancetitle = cts['name']
+            self.conformancetitle_url = cts['url']
 
             self.conformancedate = []
             for i in md.findall(util.nspath_eval(
@@ -782,14 +792,9 @@ class DQ_DataQuality(object):
                 if val is not None:
                     self.conformancedegree.append(val)
 
-            val = md.find(util.nspath_eval(
-                'gmd:lineage/gmd:LI_Lineage/gmd:statement/gco:CharacterString', namespaces))
-            self.lineage = util.testXMLValue(val)
-
-            val = md.find(util.nspath_eval('gmd:lineage/gmd:LI_Lineage/gmd:statement/gmx:Anchor', namespaces))
-            if val is not None:
-                self.lineage = util.testXMLValue(val)
-                self.lineage_url = val.attrib.get(util.nspath_eval('xlink:href', namespaces))
+            lng = testFirstCharOrAnchor(md, 'gmd:lineage/gmd:LI_Lineage/gmd:statement')
+            self.lineage = lng['name']
+            self.lineage_url = lng['url']
 
             val = md.find(util.nspath_eval(
                 'gmd:report/gmd:DQ_DomainConsistency/gmd:result/gmd:DQ_ConformanceResult/gmd:specification/gmd:CI_Citation/gmd:title/gco:CharacterString',
@@ -805,35 +810,20 @@ class DQ_DataQuality(object):
                     self.specificationdate.append(val)
 
 
-class SV_ServiceIdentification(object):
+class SV_ServiceIdentification(MD_DataIdentification):
     """ process SV_ServiceIdentification """
     def __init__(self, md=None):
+
+        super().__init__(md, 'service')
+
         if md is None:
-            self.title = None
-            self.abstract = None
-            self.contact = None
-            self.identtype = 'service'
             self.type = None
             self.version = None
             self.fees = None
-            self.bbox = None
             self.couplingtype = None
             self.operations = []
             self.operateson = []
         else:
-            val = md.find(util.nspath_eval('gmd:citation/gmd:CI_Citation/gmd:title/gco:CharacterString', namespaces))
-            self.title = util.testXMLValue(val)
-
-            val = md.find(util.nspath_eval('gmd:abstract/gco:CharacterString', namespaces))
-            self.abstract = util.testXMLValue(val)
-
-            self.contact = None
-            val = md.find(util.nspath_eval(
-                'gmd:citation/gmd:CI_Citation/gmd:citedResponsibleParty/gmd:CI_ResponsibleParty', namespaces))
-            if val is not None:
-                self.contact = CI_ResponsibleParty(val)
-
-            self.identtype = 'service'
             val = md.find(util.nspath_eval('srv:serviceType/gco:LocalName', namespaces))
             self.type = util.testXMLValue(val)
 
@@ -844,13 +834,6 @@ class SV_ServiceIdentification(object):
                 'srv:accessProperties/gmd:MD_StandardOrderProcess/gmd:fees/gco:CharacterString', namespaces))
             self.fees = util.testXMLValue(val)
 
-            val = md.find(util.nspath_eval('srv:extent/gmd:EX_Extent', namespaces))
-
-            if val is not None:
-                self.bbox = EX_Extent(val)
-            else:
-                self.bbox = None
-
             self.couplingtype = _testCodeListValue(md.find(util.nspath_eval(
                 'gmd:couplingType/gmd:SV_CouplingType', namespaces)))
 
@@ -889,18 +872,33 @@ class CI_OnlineResource(object):
         if md is None:
             self.url = None
             self.protocol = None
+            self.protocol_url = None
             self.name = None
+            self.name_url = None
             self.description = None
+            self.description_url = None
             self.function = None
+            self.applicationprofile = None
+            self.applicationprofile_url = None
         else:
             val = md.find(util.nspath_eval('gmd:linkage/gmd:URL', namespaces))
             self.url = util.testXMLValue(val)
 
-            val = md.find(util.nspath_eval('gmd:protocol/gco:CharacterString', namespaces))
-            self.protocol = util.testXMLValue(val)
+            val = testFirstCharOrAnchor(md,'gmd:protocol')
+            self.protocol = val['name']
+            self.protocol_url = val['url']
 
-            val = md.find(util.nspath_eval('gmd:name/gco:CharacterString', namespaces))
-            self.name = util.testXMLValue(val)
+            val = testFirstCharOrAnchor(md,'gmd:name')
+            self.name = val['name']
+            self.name_url = val['url']
+
+            val = testFirstCharOrAnchor(md,'gmd:description')
+            self.description = val['name']
+            self.description_url = val['url']
+
+            val = testFirstCharOrAnchor(md,'gmd:applicationProfile')
+            self.applicationprofile = val['name']
+            self.applicationprofile_url = val['url']
 
             val = md.find(util.nspath_eval('gmd:description/gco:CharacterString', namespaces))
             self.description = util.testXMLValue(val)
@@ -1000,33 +998,23 @@ class MD_ReferenceSystem(object):
     def __init__(self, md=None):
         if md is None:
             self.code = None
+            self.code_url = None
             self.codeSpace = None
+            self.codeSpace_url = None
             self.version = None
+            self.version_url = None
         else:
-            val = md.find(util.nspath_eval(
-                'gmd:referenceSystemIdentifier/gmd:RS_Identifier/gmd:code/gco:CharacterString', namespaces))
-            if val is None:
-                val = md.find(util.nspath_eval(
-                    'gmd:referenceSystemIdentifier/gmd:RS_Identifier/gmd:code/gmx:Anchor', namespaces))
-            if val is not None:
-                self.code = util.testXMLValue(val)
-            else:
-                self.code = None
+            val = testFirstCharOrAnchor(md,'gmd:referenceSystemIdentifier/gmd:RS_Identifier/gmd:code')
+            self.code_url = val['url']      
+            self.code = val['name']
 
-            val = md.find(util.nspath_eval(
-                'gmd:referenceSystemIdentifier/gmd:RS_Identifier/gmd:codeSpace/gco:CharacterString', namespaces))
-            if val is not None:
-                self.codeSpace = util.testXMLValue(val)
-            else:
-                self.codeSpace = None
-
-            val = md.find(util.nspath_eval(
-                'gmd:referenceSystemIdentifier/gmd:RS_Identifier/gmd:version/gco:CharacterString', namespaces))
-            if val is not None:
-                self.version = util.testXMLValue(val)
-            else:
-                self.version = None
+            val = testFirstCharOrAnchor(md,'gmd:referenceSystemIdentifier/gmd:RS_Identifier/gmd:codeSpace')
+            self.codeSpace_url = val['url']
+            self.codeSpace = val['name']
 
+            val = testFirstCharOrAnchor(md,'gmd:referenceSystemIdentifier/gmd:RS_Identifier/gmd:version')
+            self.version_url = val['url']
+            self.version = val['name']
 
 def _testCodeListValue(elpath):
     """ get gco:CodeListValue_Type attribute, else get text content """
