@@ -25,6 +25,7 @@ from owslib.util import (openURL, testXMLValue, extract_xml_list,
                          bind_url, nspath_eval, Authentication)
 from owslib.fgdc import Metadata
 from owslib.iso import MD_Metadata
+from owslib.iso3 import MD_Metadata as MD_Metadata3  # ISO 19115 Part 3 XML
 from owslib.map.common import WMSCapabilitiesReader, AbstractContentMetadata
 from owslib.namespaces import Namespaces
 
@@ -68,7 +69,7 @@ class WebMapService_1_1_1(object):
         # Authentication handled by Reader
         reader = WMSCapabilitiesReader(
             self.version, url=self.url, headers=headers, auth=self.auth)
-        if xml:  # read from stored xml
+        if xml is not None:  # read from stored xml
             self._capabilities = reader.readString(xml)
         else:  # read from server
             self._capabilities = reader.read(self.url, timeout=self.timeout)
@@ -171,7 +172,7 @@ class WebMapService_1_1_1(object):
         request['height'] = str(size[1])
 
         request['srs'] = str(srs)
-        request['bbox'] = ','.join([repr(x) for x in bbox])
+        request['bbox'] = ','.join([str(x) for x in bbox])
         request['format'] = str(format)
         request['transparent'] = str(transparent).upper()
         request['exceptions'] = str(exceptions)
@@ -617,6 +618,13 @@ class ContentMetadata(AbstractContentMetadata):
                         if mdelem is not None:
                             metadataUrl['metadata'] = MD_Metadata(mdelem)
                             continue
+                        else:  # ISO 19115 Part 3 XML
+                            mdelem = MD_Metadata3.find_start(doc)
+                            if mdelem is not None:
+                                metadataUrl["metadata"] = MD_Metadata3(mdelem)
+                            else:
+                                metadataUrl["metadata"] = None
+                            continue
                 except Exception:
                     metadataUrl['metadata'] = None
 
