@@ -703,8 +703,14 @@ class ContentMetadata(AbstractContentMetadata):
                         metadataUrl['metadata'] = Metadata(mdelem)
                         continue
 
-                    mdelem = doc.find('.//' + nspath_eval('gmd:MD_Metadata', n.get_namespaces(['gmd']))) \
-                        or doc.find('.//' + nspath_eval('gmi:MI_Metadata', n.get_namespaces(['gmi'])))
+                    mdelem = doc.find(
+                        './/' + nspath_eval('gmd:MD_Metadata', n.get_namespaces(['gmd']))
+                    )
+                    if mdelem is None:
+                        doc.find(
+                            './/' + nspath_eval('gmi:MI_Metadata', n.get_namespaces(['gmi']))
+                        )
+
                     if mdelem is not None:
                         metadataUrl['metadata'] = MD_Metadata(mdelem)
                         continue
