@@ -1,6 +1,6 @@
 # -*- coding: ISO-8859-15 -*-
 # =============================================================================
-# Copyright (c) 2008 Tom Kralidis
+# Copyright (c) 2024 Tom Kralidis
 #
 # Authors : Tom Kralidis <tomkralidis@gmail.com>
 #
@@ -11,11 +11,10 @@ import os
 import sys
 from collections import OrderedDict
 from dateutil import parser
-from datetime import datetime, timedelta
-import pytz
+from datetime import datetime, timedelta, timezone
 from owslib.etree import etree, ParseError
 from owslib.namespaces import Namespaces
-from urllib.parse import urlsplit, urlencode, urlparse, parse_qs, urlunparse, parse_qsl, unquote
+from urllib.parse import urlsplit, urlencode, urlparse, parse_qs, urlunparse, parse_qsl
 import copy
 
 from io import StringIO, BytesIO
@@ -197,7 +196,7 @@ def openURL(url_base, data=None, method='Get', cookies=None, username=None, pass
         rkwargs['data'] = data
 
     elif method.lower() == 'get':
-        rkwargs['params'] = unquote(data) if data else None
+        rkwargs['params'] = data
 
     else:
         raise ValueError("Unknown method ('%s'), expected 'get' or 'post'" % method)
@@ -207,7 +206,7 @@ def openURL(url_base, data=None, method='Get', cookies=None, username=None, pass
 
     req = requests.request(method.upper(), url_base, headers=headers, **rkwargs)
 
-    if req.status_code in [400, 401]:
+    if req.status_code in [400, 401, 403]:
         raise ServiceException(req.text)
 
     if req.status_code in [404, 500, 502, 503, 504]:    # add more if needed
@@ -277,11 +276,8 @@ def nspath_eval(xpath, namespaces):
 
 def cleanup_namespaces(element):
     """ Remove unused namespaces from an element """
-    if etree.__name__ == 'lxml.etree':
-        etree.cleanup_namespaces(element)
-        return element
-    else:
-        return etree.fromstring(etree.tostring(element))
+    etree.cleanup_namespaces(element)
+    return element
 
 
 def add_namespaces(root, ns_keys):
@@ -292,35 +288,21 @@ def add_namespaces(root, ns_keys):
 
     ns_keys = [(x, namespaces.get_namespace(x)) for x in ns_keys]
 
-    if etree.__name__ != 'lxml.etree':
-        # We can just add more namespaces when not using lxml.
-        # We can't re-add an existing namespaces.  Get a list of current
-        # namespaces in use
-        existing_namespaces = set()
-        for elem in root.iter():
-            if elem.tag[0] == "{":
-                uri, tag = elem.tag[1:].split("}")
-                existing_namespaces.add(namespaces.get_namespace_from_url(uri))
-        for key, link in ns_keys:
-            if link is not None and key not in existing_namespaces:
-                root.set("xmlns:%s" % key, link)
-        return root
-    else:
-        # lxml does not support setting xmlns attributes
-        # Update the elements nsmap with new namespaces
-        new_map = root.nsmap
-        for key, link in ns_keys:
-            if link is not None:
-                new_map[key] = link
-        # Recreate the root element with updated nsmap
-        new_root = etree.Element(root.tag, nsmap=new_map)
-        # Carry over attributes
-        for a, v in list(root.items()):
-            new_root.set(a, v)
-        # Carry over children
-        for child in root:
-            new_root.append(deepcopy(child))
-        return new_root
+    # lxml does not support setting xmlns attributes
+    # Update the elements nsmap with new namespaces
+    new_map = root.nsmap
+    for key, link in ns_keys:
+        if link is not None:
+            new_map[key] = link
+    # Recreate the root element with updated nsmap
+    new_root = etree.Element(root.tag, nsmap=new_map)
+    # Carry over attributes
+    for a, v in list(root.items()):
+        new_root.set(a, v)
+    # Carry over children
+    for child in root:
+        new_root.append(deepcopy(child))
+    return new_root
 
 
 def getXMLInteger(elem, tag):
@@ -343,6 +325,43 @@ def getXMLInteger(elem, tag):
     return int(e.text.strip())
 
 
+def getXMLTree(rsp: ResponseWrapper) -> etree:
+    """
+    Parse a response into an XML elementtree instance
+    and raise a ValueError if the server returns a
+    non-XML response. The response may contain a useful
+    error message from the server.
+
+    Parameters
+    ----------
+
+    @param rsp: the ResponseWrapper for the XML request
+    """
+
+    raw_text = strip_bom(rsp.read())
+    et = etree.fromstring(raw_text)
+
+    # check for response type - if it is not xml then raise an error
+    # if the server doesn't provide a Content-Type then assume xml
+    content_type = rsp.info().get('Content-Type', 'text/xml')
+    url = rsp.geturl()
+
+    has_xml_tag = raw_text.lstrip().startswith(b'<?xml')
+
+    xml_types = ['text/xml', 'application/xml', 'application/vnd.ogc.wms_xml']
+    if not any(xt in content_type.lower() for xt in xml_types) and not has_xml_tag:
+        html_body = et.find('BODY')  # note this is case-sensitive
+        if html_body is not None and len(html_body.text) > 0:
+            response_text = html_body.text.strip("\n")
+        else:
+            response_text = raw_text
+
+        raise ValueError("%s responded with Content-Type '%s': '%s'" %
+                         (url, content_type, response_text))
+
+    return et
+
+
 def testXMLValue(val, attrib=False):
     """
 
@@ -384,7 +403,7 @@ def testXMLAttribute(element, attribute):
     return None
 
 
-def http_post(url=None, request=None, lang='en-US', timeout=10, username=None, password=None, auth=None):
+def http_post(url=None, request=None, lang='en-US', timeout=10, username=None, password=None, auth=None, headers=None):
     """
 
     Invoke an HTTP POST request
@@ -396,6 +415,8 @@ def http_post(url=None, request=None, lang='en-US', timeout=10, username=None, p
     - request: the request message
     - lang: the language
     - timeout: timeout in seconds
+    - auth: owslib.util.Auth instance
+    - headers: HTTP headers to send with requests
 
     """
 
@@ -404,8 +425,8 @@ def http_post(url=None, request=None, lang='en-US', timeout=10, username=None, p
 
     u = urlsplit(url)
 
-    headers = {
-        'User-Agent': 'OWSLib (https://geopython.github.io/OWSLib)',
+    headers_ = {
+        'User-Agent': 'OWSLib (https://owslib.readthedocs.io',
         'Content-type': 'text/xml',
         'Accept': 'text/xml,application/xml',
         'Accept-Language': lang,
@@ -413,6 +434,13 @@ def http_post(url=None, request=None, lang='en-US', timeout=10, username=None, p
         'Host': u.netloc,
     }
 
+    if headers:
+        headers_.update(headers)
+
+    if isinstance(request, dict):
+        headers_['Content-type'] = 'application/json'
+        headers_.pop('Accept')
+
     rkwargs = {}
 
     if auth:
@@ -429,11 +457,13 @@ def http_post(url=None, request=None, lang='en-US', timeout=10, username=None, p
     rkwargs['verify'] = auth.verify
     rkwargs['cert'] = auth.cert
 
-    up = requests.post(url, request, headers=headers, **rkwargs)
-    return up.content
+    if not isinstance(request, dict):
+        return requests.post(url, request, headers=headers_, **rkwargs)
+    else:
+        return requests.post(url, json=request, headers=headers_, **rkwargs)
 
 
-def http_get(*args, **kwargs):
+def http_prepare(*args, **kwargs):
     # Copy input kwargs so the dict can be modified
     rkwargs = copy.deepcopy(kwargs)
 
@@ -464,9 +494,33 @@ def http_get(*args, **kwargs):
         rkwargs.setdefault('auth', None)
     rkwargs.setdefault('cert', rkwargs.get('cert'))
     rkwargs.setdefault('verify', rkwargs.get('verify', True))
+
+    return rkwargs
+
+
+def http_get(*args, **kwargs):
+    rkwargs = http_prepare(*args, **kwargs)
     return requests.get(*args, **rkwargs)
 
 
+def http_put(*args, **kwargs):
+    rkwargs = http_prepare(*args, **kwargs)
+
+    if 'data' in kwargs:
+        if isinstance(kwargs['data'], dict):
+            rkwargs['json'] = kwargs['data']
+            rkwargs.pop('data')
+        else:
+            rkwargs['data'] = kwargs['data']
+
+    return requests.put(*args, **rkwargs)
+
+
+def http_delete(*args, **kwargs):
+    rkwargs = http_prepare(*args, **kwargs)
+    return requests.delete(*args, **rkwargs)
+
+
 def element_to_string(element, encoding=None, xml_declaration=False):
     """
     Returns a string from a XML object
@@ -484,21 +538,14 @@ def element_to_string(element, encoding=None, xml_declaration=False):
     if encoding is None:
         encoding = "ISO-8859-1"
 
-    if etree.__name__ == 'lxml.etree':
-        if xml_declaration:
-            if encoding in ['unicode', 'utf-8']:
-                output = '<?xml version="1.0" encoding="utf-8" standalone="no"?>\n{}'.format(
-                    etree.tostring(element, encoding='unicode'))
-            else:
-                output = etree.tostring(element, encoding=encoding, xml_declaration=True)
+    if xml_declaration:
+        if encoding in ['unicode', 'utf-8']:
+            output = '<?xml version="1.0" encoding="utf-8" standalone="no"?>\n{}'.format(
+                etree.tostring(element, encoding='unicode'))
         else:
-            output = etree.tostring(element)
+            output = etree.tostring(element, encoding=encoding, xml_declaration=True)
     else:
-        if xml_declaration:
-            output = '<?xml version="1.0" encoding="{}" standalone="no"?>\n{}'.format(
-                encoding, etree.tostring(element, encoding=encoding))
-        else:
-            output = etree.tostring(element)
+        output = etree.tostring(element)
 
     return output
 
@@ -555,7 +602,7 @@ def getNamespace(element):
         return ""
 
 
-def build_get_url(base_url, params, overwrite=False):
+def build_get_url(base_url, params, overwrite=False, doseq=False):
     ''' Utility function to build a full HTTP GET URL from the service base URL and a dictionary of HTTP parameters.
 
     TODO: handle parameters case-insensitive?
@@ -587,14 +634,14 @@ def build_get_url(base_url, params, overwrite=False):
         if key not in pars:
             qs.append((key, value))
 
-    urlqs = urlencode(tuple(qs))
+    urlqs = urlencode(tuple(qs), doseq=doseq)
     return base_url.split('?')[0] + '?' + urlqs
 
 
 def dump(obj, prefix=''):
     '''Utility function to print to standard output a generic object with all its attributes.'''
 
-    print(("{} {}.{} : {}".format(prefix, obj.__module__, obj.__class__.__name__, obj.__dict__)))
+    return "{} {}.{} : {}".format(prefix, obj.__module__, obj.__class__.__name__, obj.__dict__)
 
 
 def getTypedValue(data_type, value):
@@ -637,8 +684,7 @@ Would be 2006-07-27T21:10:00Z, not 'now'
     except Exception:
         att = testXMLValue(element.attrib.get('indeterminatePosition'), True)
         if att and att == 'now':
-            dt = datetime.utcnow()
-            dt.replace(tzinfo=pytz.utc)
+            dt = datetime.utcnow().replace(tzinfo=timezone.utc)
         else:
             dt = None
     return dt
@@ -734,29 +780,6 @@ def bind_url(url):
     return '%s%s' % (url, binder)
 
 
-import logging
-# Null logging handler
-NullHandler = logging.NullHandler
-
-log = logging.getLogger('owslib')
-log.addHandler(NullHandler())
-
-
-def which_etree():
-    """decipher which etree library is being used by OWSLib"""
-
-    which_etree = None
-
-    if 'lxml' in etree.__file__:
-        which_etree = 'lxml.etree'
-    elif 'xml/etree' in etree.__file__:
-        which_etree = 'xml.etree'
-    elif 'elementree' in etree.__file__:
-        which_etree = 'elementtree.ElementTree'
-
-    return which_etree
-
-
 def findall(root, xpath, attribute_name=None, attribute_value=None):
     """Find elements recursively from given root element based on
     xpath and possibly given attribute
@@ -790,7 +813,7 @@ def datetime_from_iso(iso):
 
 def datetime_from_ansi(ansi):
     """Converts an ansiDate (expressed as a number = the nuber of days since the datum origin of ansi)
-    to a python datetime object.
+    to a Python datetime object.
     """
 
     datumOrigin = datetime(1600, 12, 31, 0, 0, 0)
