@@ -34,7 +34,7 @@ import warnings
 from urllib.parse import (urlencode, urlparse, urlunparse, parse_qs,
                           ParseResult)
 from .etree import etree
-from .util import clean_ows_url, testXMLValue, getXMLInteger, Authentication, openURL
+from .util import clean_ows_url, testXMLValue, getXMLInteger, Authentication, openURL, getXMLTree, nspath
 from .fgdc import Metadata
 from .iso import MD_Metadata
 from .ows import ServiceProvider, ServiceIdentification, OperationsMetadata
@@ -78,6 +78,14 @@ _SERVICE_METADATA_URL_TAG = _WMTS_NS + 'ServiceMetadataURL'
 _STYLE_TAG = _WMTS_NS + 'Style'
 _STYLE_LEGEND_URL = _WMTS_NS + 'LegendURL'
 
+# Table 9, page 22-23, Parts of Dimensions data structure
+_DIMENSION_TAG = _WMTS_NS + 'Dimension'
+_UOM_TAG = _OWS_NS + 'UOM'
+_DIMENSION_UNIT_SYMBOL_TAG = _WMTS_NS + 'UnitSymbol'
+_DIMENSION_DEFAULT_TAG = _WMTS_NS + 'Default'
+_DIMENSION_CURRENT_TAG = _WMTS_NS + 'Current'
+_DIMENSION_VALUE_TAG = _WMTS_NS + 'Value'
+
 _THEME_TAG = _WMTS_NS + 'Theme'
 _THEMES_TAG = _WMTS_NS + 'Themes'
 _TILE_HEIGHT_TAG = _WMTS_NS + 'TileHeight'
@@ -172,7 +180,7 @@ class WebMapTileService(object):
         # Authentication handled by Reader
         reader = WMTSCapabilitiesReader(
             self.version, url=self.url, headers=self.headers, auth=self.auth)
-        if xml:  # read from stored xml
+        if xml is not None:  # read from stored xml
             self._capabilities = reader.readString(xml)
         else:  # read from server
             self._capabilities = reader.read(self.url, self.vendor_kwargs)
@@ -219,7 +227,8 @@ class WebMapTileService(object):
         #  REST only WMTS does not have any Operations
         if serviceop is not None:
             for elem in serviceop[:]:
-                self.operations.append(OperationsMetadata(elem))
+                if elem.tag != nspath('ExtendedCapabilities'):
+                    self.operations.append(OperationsMetadata(elem))
 
         # serviceContents metadata: our assumption is that services use
         # a top-level layer as a metadata organizer, nothing more.
@@ -233,8 +242,9 @@ class WebMapTileService(object):
                     parse_remote_metadata=parse_remote_metadata)
                 if cm.id:
                     if cm.id in self.contents:
-                        raise KeyError('Content metadata for layer "%s" '
-                                       'already exists' % cm.id)
+                        msg = ('Content metadata for layer "%s" '
+                               'already exists' % cm.id)
+                        warnings.warn(msg, RuntimeWarning)
                     self.contents[cm.id] = cm
                 gather_layers(elem, cm)
         gather_layers(caps, None)
@@ -244,8 +254,9 @@ class WebMapTileService(object):
             tms = TileMatrixSet(elem)
             if tms.identifier:
                 if tms.identifier in self.tilematrixsets:
-                    raise KeyError('TileMatrixSet with identifier "%s" '
-                                   'already exists' % tms.identifier)
+                    msg = ('TileMatrixSet with identifier "%s" '
+                           'already exists' % tms.identifier)
+                    warnings.warn(msg, RuntimeWarning)
                 self.tilematrixsets[tms.identifier] = tms
 
         self.themes = {}
@@ -253,8 +264,9 @@ class WebMapTileService(object):
             theme = Theme(elem)
             if theme.identifier:
                 if theme.identifier in self.themes:
-                    raise KeyError('Theme with identifier "%s" already exists'
-                                   % theme.identifier)
+                    msg = ('Theme with identifier "%s" already exists'
+                           % theme.identifier)
+                    warnings.warn(msg, RuntimeWarning)
                 self.themes[theme.identifier] = theme
 
         serviceMetadataURL = self._capabilities.find(_SERVICE_METADATA_URL_TAG)
@@ -511,8 +523,9 @@ class TileMatrixSet(object):
             tm = TileMatrix(tilematrix)
             if tm.identifier:
                 if tm.identifier in self.tilematrix:
-                    raise KeyError('TileMatrix with identifier "%s" '
-                                   'already exists' % tm.identifier)
+                    msg = ('TileMatrix with identifier "%s" '
+                           'already exists' % tm.identifier)
+                    warnings.warn(msg, RuntimeWarning)
                 self.tilematrix[tm.identifier] = tm
 
 
@@ -740,9 +753,9 @@ class ContentMetadata:
         for tmsl in tile_matrix_set_links:
             if tmsl.tilematrixset:
                 if tmsl.tilematrixset in self.tilematrixsetlinks:
-                    raise KeyError('TileMatrixSetLink with tilematrixset "%s"'
-                                   ' already exists' %
-                                   tmsl.tilematrixset)
+                    msg = ('TileMatrixSetLink with tilematrixset "%s"'
+                           ' already exists' % tmsl.tilematrixset)
+                    warnings.warn(msg, RuntimeWarning)
                 self.tilematrixsetlinks[tmsl.tilematrixset] = tmsl
 
         self.resourceURLs = []
@@ -793,6 +806,65 @@ class ContentMetadata:
                          _KEYWORDS_TAG + '/' + _KEYWORD_TAG)]
         self.infoformats = [f.text for f in elem.findall(_INFO_FORMAT_TAG)]
 
+        self.dimensions = {}
+        for dim in elem.findall(_DIMENSION_TAG):
+            dimension = {}
+
+            identifier = dim.find(_IDENTIFIER_TAG)
+            if identifier is None:
+                # mandatory parameter
+                raise ValueError('%s missing identifier' % (dim,))
+            if identifier.text in self.dimensions:
+                # domain identifier SHALL be unique
+                warnings.warn('%s identifier duplicated, taking first occurence' % (dim,))
+                continue
+
+            values = [f.text for f in dim.findall(_DIMENSION_VALUE_TAG)]
+            if len(values) == 0:
+                raise ValueError(
+                    '%s list of values can not be empty' % (dim,)
+                )
+            dimension['values'] = values
+
+            title = dim.find(_TITLE_TAG)
+            if title is not None:
+                dimension['title'] = title.text
+
+            abstract = dim.find(_ABSTRACT_TAG)
+            if abstract is not None:
+                dimension['abstract'] = abstract.text
+
+            keywords = [
+                f.text for f in dim.findall(_KEYWORDS_TAG + '/' + _KEYWORD_TAG)
+            ]
+            if keywords:
+                dimension['keywords'] = keywords
+
+            uom = dim.find(_UOM_TAG)
+            if uom is not None:
+                dimension['UOM'] = uom.text
+
+            unit_symbol = dim.find(_DIMENSION_UNIT_SYMBOL_TAG)
+            if unit_symbol is not None:
+                dimension['unit_symbol'] = unit_symbol.text
+
+            default_value = dim.find(_DIMENSION_DEFAULT_TAG)
+            if default_value in ['default', 'current', '', None]:
+                # mandatory parameter
+                raise ValueError(
+                    '%s default value must not be empty or \'default\' or \'current\''
+                    % (dim,)
+                )
+            dimension['default'] = default_value.text
+
+            current = dim.find(_DIMENSION_CURRENT_TAG)
+            if current and current.text == 'true':
+                dimension['current'] = True
+            else:
+                dimension['current'] = False
+
+            self.dimensions[identifier.text] = dimension
+
         self.layers = []
         for child in elem.findall(_LAYER_TAG):
             self.layers.append(ContentMetadata(child, self))
@@ -862,7 +934,7 @@ class WMTSCapabilitiesReader:
         # now split it up again to use the generic openURL function...
         spliturl = getcaprequest.split('?')
         u = openURL(spliturl[0], spliturl[1], method='Get', headers=self.headers, auth=self.auth)
-        return etree.fromstring(u.read())
+        return getXMLTree(u)
 
     def readString(self, st):
         """Parse a WMTS capabilities document, returning an elementtree instance
